; ------------------------------------------------------------------------------------------------

FUNCTION idl_rpc_client::RPCInit, HOSTNAME = hostname

IF NOT KEYWORD_SET (hostname) THEN BEGIN

   PRINT, 'A hostname must be specified in order to use this method.'

   RETURN, 0

ENDIF

self.hostname = hostname

status = self.rpc_client_obj->open_service (PROG     = self.idl_rpc_id,          $
                                            VERS     = self.idl_rpc_version,     $
                                            HOSTNAME = self.hostname             $
                                            )

IF NOT status THEN BEGIN

   PRINT, 'Failed to connect to IDL server on host: ', self.hostname, '.'

   RETURN, 0

ENDIF

self.connect_flag = 1

RETURN, 1

END

; ------------------------------------------------------------------------------------------------

FUNCTION idl_rpc_client::RPCOutputCapture, capture_lines

IF NOT self.connect_flag EQ 0 THEN BEGIN

   PRINT, 'Client must be connected to IDL Server before using this method.'

   RETURN, 0

ENDIF

status = self.rpc_client_obj->call_service (SVR_PROC   = self.IDL_RPC_OUT_CAPTURE,      $
                                       DATA_VALUE = capture_lines)

IF NOT status THEN BEGIN

   PRINT, 'Attempt to execute procedure IDL_RPC_OUT_CAPTURE failed.'

   RETURN, 0

ENDIF

RETURN, 1

END

; ------------------------------------------------------------------------------------------------

FUNCTION idl_rpc_client::RPCOutputGetStr, LINE = line, FIRST = first, FLAGS = flags

IF NOT self.connect_flag EQ 0 THEN BEGIN

   PRINT, 'Client must be connected to IDL Server before using this method.'

   RETURN, 0

ENDIF

IF KEYWORD_SET (first) THEN first = 1 ELSE first = 0

status = self.rpc_client_obj->call_service (SVR_PROC = self.IDL_RPC_OUT_STR, DATA_VALUE = first)


IF NOT status THEN BEGIN

   PRINT, 'Attempt to execute procedure IDL_RPC_OUT_STR failed.'

   RETURN, 0

ENDIF

reply_data = self.rpc_client_obj->get_reply_data ()

line  = reply_data.buf
flags = reply_data.flags

; PRINT, 'flags = ', flags, FORMAT = '(A, Z0)'

IF flags EQ -1 THEN BEGIN

   self.buffer_empty_flag = 1
   self.last_line_flag    = 0

ENDIF ELSE IF flags AND '01'X EQ 0 THEN BEGIN

   self.buffer_empty_flag = 0
   self.last_line_flag    = 1

ENDIF ELSE BEGIN

   self.buffer_empty_flag = 0
   self.last_line_flag    = 0

ENDELSE


RETURN, 1

END

; ------------------------------------------------------------------------------------------------

FUNCTION idl_rpc_client::RPCExecuteStr, idlcmd, ERROR = error

IF NOT self.connect_flag EQ 0 THEN BEGIN

   PRINT, 'Client must be connected to IDL Server before using this method.'

   RETURN, 0

ENDIF

IF NOT KEYWORD_SET (idlcmd) THEN BEGIN

   PRINT, 'A string must be passed as the input to this method.'

   RETURN, 0

ENDIF

status = self.rpc_client_obj->call_service (SVR_PROC = self.IDL_RPC_EXE_STR, DATA_VALUE = idlcmd)

IF NOT status THEN BEGIN

   PRINT, 'Attempt to execute procedure IDL_RPC_EXE_STR failed.'

   RETURN, 0

ENDIF

retval = self.rpc_client_obj->get_reply_data ()

IF retval NE 0 AND N_ELEMENTS (error) NE 0 THEN error = retval

; IF retval EQ 0 THEN RETURN, 1 ELSE RETURN, 0

; Cludge for now, until I figure out how to interpret the retrun value!

RETURN, status

END

; ------------------------------------------------------------------------------------------------

FUNCTION idl_rpc_client::RPCCleanup, KILL_SERVER = kill_server

IF NOT self.connect_flag EQ 0 THEN BEGIN

   PRINT, 'Client must be connected to IDL Server before using this method.'

   RETURN, 0

ENDIF

IF KEYWORD_SET (kill_server) THEN BEGIN

   IF NOT self.rpc_client_obj->call_service (SVR_PROC = self.IDL_RPC_CLEANUP) THEN BEGIN

      PRINT, 'Attempt to execute procedure IDL_RPC_CLEANUP failed.'

   ENDIF

ENDIF


status = self.rpc_client_obj->close_service ()

self.connect_flag = 0

IF NOT status THEN BEGIN

   PRINT, 'Error while closing connection to IDL RPC Server.'

ENDIF

RETURN, status

END

; ------------------------------------------------------------------------------------------------

FUNCTION idl_rpc_client::init, IDL_RPC_ID = idl_rpc_id, IDL_RPC_VERSION = idl_rpc_version

self.IDL_RPC_MAX_STRLEN      = 512
self.IDL_RPC_DEFAULT_ID      = '2010CAFE'X
self.IDL_RPC_DEFAULT_VERSION = 1
self.IDL_RPC_SET_VAR         = '10'X
self.IDL_RPC_GET_VAR         = '20'X
self.IDL_RPC_SET_MAIN_VAR    = '30'X
self.IDL_RPC_GET_MAIN_VAR    = '40'X
self.IDL_RPC_EXE_STR         = '50'X
self.IDL_RPC_OUT_CAPTURE     = '60'X
self.IDL_RPC_OUT_STR         = '70'X
self.IDL_RPC_CLEANUP         = '90'X

self.connect_flag = 0
self.buffer_empty_flag = 0
self.last_line_flag = 0


IF NOT KEYWORD_SET (idl_rpc_id)      THEN idl_rpc_id = self.IDL_RPC_DEFAULT_ID
IF NOT KEYWORD_SET (idl_rpc_version) THEN idl_rpc_version = self.IDL_RPC_DEFAULT_VERSION

self.idl_rpc_id      = idl_rpc_id
self.idl_rpc_version = idl_rpc_version

self.rpc_client_obj = OBJ_NEW ('RPC_CLIENT')

IF NOT OBJ_VALID (self.rpc_client_obj) THEN RETURN, 0

RETURN, 1

END

; ------------------------------------------------------------------------------------------------

PRO idl_rpc_client::cleanup

IF self.connect_flag THEN status = self->RPCCleanup ()

IF OBJ_VALID (self.rpc_client_obj) THEN OBJ_DESTROY, self.rpc_client_obj

RETURN

END

; ------------------------------------------------------------------------------------------------

PRO idl_rpc_client__define

struct = {IDL_RPC_CLIENT,                                  $
          connect_flag:                 0L,                $
          buffer_empty_flag:            0L,                $
          last_line_flag:               0L,                $
          hostname:                     '',                $
          idl_rpc_id:                   0L,                $
          idl_rpc_version:              0L,                $
          rpc_client_obj:               OBJ_NEW (),        $
          IDL_RPC_MAX_STRLEN:           0L,                $
          IDL_RPC_DEFAULT_ID:           0L,                $
          IDL_RPC_DEFAULT_VERSION:      0L,                $
          IDL_RPC_SET_VAR:              0L,                $
          IDL_RPC_GET_VAR:              0L,                $
          IDL_RPC_SET_MAIN_VAR:         0L,                $
          IDL_RPC_GET_MAIN_VAR:         0L,                $
          IDL_RPC_EXE_STR:              0L,                $
          IDL_RPC_OUT_CAPTURE:          0L,                $
          IDL_RPC_OUT_STR:              0L,                $
          IDL_RPC_CLEANUP:              0L                 $
         }

RETURN

END