;+
; Project     : HESSI
;
; Name        : SOCK_COPY
;
; Purpose     : copy file via HTTP sockets
;
; Category    : utility system sockets
;
; Syntax      : IDL> sock_copy,url_file,outdir=outdir
;                   
; Inputs      : URL_FILE = remote file name to copy with URL path
;
; Outputs     : None
;
; Keywords    : OUT_DIR = output directory to copy file
;               ERR   = string error message
;
; Example     : 
;
; IDL> f='smmdac.nascom.nasa.gov/synop_data/kanz/kanz_halph_fd_20000114_1016.fts
; IDL> sock_copy,f
;
; History     : 27-Dec-2001,  D.M. Zarro (EITI/GSFC)  Written
;
; Contact     : DZARRO@SOLAR.STANFORD.EDU
;-

pro sock_copy,url,err=err,_ref_extra=extra

common sock_copy,http

err=''

if is_blank(url) then begin
 err='missing input URL'
 pr_syntax,'sock_copy,url'
 return
endif

;-- check if using FTP

is_ftp=stregex(url,'ftp://',/bool)
if is_ftp[0] then begin
 read_ftp,url,/file,err=err,_extra=extra
 return
endif

if not obj_valid(http) then begin
 http=obj_new('http',err=err)
 if err ne '' then return
endif

http->hset,_extra=extra
for i=0,n_elements(url)-1 do http->copy,url[i],_extra=extra,err=err

http->clear

return
end


