;+
; Project     : HESSI
;                  
; Name        : WRITE_DIR
;               
; Purpose     : platform/OS independent check if directory as write permission
;                             
; Category    : system utility
;               
; Explanation : uses 'openw'
;               
; Syntax      : IDL> a=write_dir(name)
;    
; Inputs      : NAME = directory name to check
;               
; Outputs     : 1/0 if success/failure
;
; Keywords    : OUT = translated name of input directory
;             
; Restrictions: Probably works in Windows
;               
; History     : Written,  6-June-1999, Zarro (SM&A/GSFC)
;               Modified, 29-Nov-1999, Zarro - added call to TEST_OPEN
;               Modfiied, 13-Mar-2000, Zarro - vectorized
;
; Contact     : dzarro@solar.stanford.edu
;-    

function write_dir,name,out=out,_extra=extra

np=n_elements(name)
if np eq 0 then return,0b
out=strarr(np)
access=bytarr(np)
for i=0,np-1 do begin
 if is_dir(name[i],out=tout,_extra=extra) then begin
  access[i]=test_open(tout,/write,/quiet)
  out[i]=tout
 endif
endfor

if np eq 1 then begin
 access=access[0]
 out=out[0]
endif

return,access

end
